body {
    margin: 0;
    padding: 0;
    text-align: center; /* !!! */
}

.kl-console {
    margin:0 auto;
    text-align:left;
    width:800px;
    height:400px;
    background-color:black;
    color:white;
    overflow: auto;
    font-size:1.2em;
    font-family:monospace;
}

.kl-console-half-size {
    height:200px;
    
}

.cem {
    color:yellow;
}

.cerr {
    color:red;
}