<html>
  <head>
  <!-- Kaylee client library -->
  <script src="/static/kaylee/js/kaylee.js" type="text/javascript"></script>

  <!-- Kaylee Echo -->
  <link href="/static/kaylee/css/klconsole.css" rel="stylesheet" />
  <script src="/static/kaylee/js/jquery.min.js" type="text/javascript"></script>
  <script src="/static/kaylee/js/kldebug.js" type="text/javascript"></script>

  <title>Kaylee Console</title>
  </head>

  <body></body>
</html>
