# -*- coding: utf-8 -*-
"""
    kaylee.contrib.frontends
    ~~~~~~~~~~~~~~~~~~~~~~~~

    The sub-package contains various frond-ends integration code.

    :copyright: (c) 2013 by Zaur Nasibov.
    :license: MIT, see LICENSE for more details.
"""
